//
//  lkfcmasstests.swift
//  lkfcTests
//
//  Created by Alexander Holzer on 22.09.19.
//  Copyright © 2019 Alexander Holzer. All rights reserved.
//

import XCTest
@testable import lkfc

class MassTestConfig {
    let path: URL
    let stamm: String
    let model: UInt32
    let year: UInt32
    var compare = LKFBatchDifferenceMode.print
    var caseStat : LKFCaseStat?
    var removeBM = false
    var contextFlags : Int32?
    var errorFiles = false
    var packageSize : Int?
    
    static var enableMassTest = true

    init (url: URL, stamm:String, model: UInt32, year: UInt32) {
        self.path = url
        self.stamm = stamm
        self.model = model
        self.year = year
    }
    
    
    convenience init (source: MassTestConfig) {
        self.init (url: source.path, stamm: source.stamm, model: source.model, year: source.year)
        removeBM = source.removeBM
        contextFlags = source.contextFlags
        errorFiles = source.errorFiles
    }

    
    convenience init (path: String, stamm:String, model: UInt32, year: UInt32) {
        self.init (url: URL(fileURLWithPath: path, isDirectory: true), stamm: stamm, model: model, year: year)
    }
    
    convenience init(path: String, year: UInt32) {
        self.init(url: URL(fileURLWithPath: path, isDirectory: true), stamm: path, model: year, year: year)
    }
    
    convenience init(path: String, model: UInt32, year: UInt32) {
        self.init(url: URL(fileURLWithPath: path, isDirectory: true), stamm: path, model: model, year: year)
    }
    
    func with (removeBM: Bool) -> MassTestConfig {
        let mtc = MassTestConfig(source:self)
        mtc.removeBM = removeBM
        return mtc
    }
    
    func with (errorFiles: Bool) -> MassTestConfig {
        let mtc = MassTestConfig(source:self)
        mtc.errorFiles = errorFiles
        return mtc
    }
    
    func with (contextFlags : Int32) -> MassTestConfig {
        let mtc = MassTestConfig(source:self)
        mtc.contextFlags = contextFlags
        return mtc
    }
    
    func test() {
        test (testOutput: false, diff: compare)
    }
    
    func test(testOutput: Bool, diff: LKFBatchDifferenceMode) {
        let files : [URL]
        do {
            files = try FileManager.default.contentsOfDirectory(at: path, includingPropertiesForKeys: nil, options: [])
        }
        catch {
            print (error)
            assert(false, "cannot read input files")
            return
        }
        
        let batchParameters = LKFBatchFileExecutionParametersForTest (outPath: lkfcTests.outPath)
        batchParameters.unloadStamm = true
        batchParameters.rootPath = stamm
        batchParameters.createTestOutput = testOutput
        batchParameters.compareMode = diff
        batchParameters.year = year
        batchParameters.model = model
        batchParameters.errorFiles = errorFiles
        batchParameters.includeStat = false
        batchParameters.removeBM = removeBM
        batchParameters.contextFlags = contextFlags
        
        var failCount = 0
        var lastFailure : String? = nil
               
        for inputFile in files {

            let lpc = inputFile.lastPathComponent
            
            if lpc.starts(with: ".") || !lpc.contains(".") {
                continue
            }
            
            print ("******* testdata: \(inputFile)")
                   
            let execution = LKFBatchFileExecution(file: inputFile, parameters: batchParameters)
            
            if let packageSize = packageSize {
                execution.packageSize = packageSize
            }
            
            let result = execution.execute()
            
            if let caseStat = execution.caseStat {
                if self.caseStat == nil {
                    self.caseStat = caseStat
                }
                else {
                    self.caseStat = self.caseStat!.adding (caseStat)
                }
            }
                   
            if let failure = result as? LKFBatchFileExecutionResultFailed {
                lastFailure = failure.failureTextWithFile
                failCount += 1
            }
        }
               
        if failCount > 0 {
            if failCount == 1 {
                XCTAssert(false, lastFailure!)
            }
            else {
                XCTAssert(false, "\(failCount) failed, last = \(lastFailure!)")
            }
        }
    }
}

class lkfcmasstests: XCTestCase {
    
    static let configs : [MassTestConfig] = {
        LKFContext.unloadStamm()
        var configs = [MassTestConfig]()
/*
        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)in2019", year: 2019))
        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)in2020", model: 2020, year: 2019))
 
        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)ina2020", year: 2020))
 
        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)ina2021", year: 2021))
        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)in2021", model: 2021, year: 2019))
 */
        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)in2022", model: 2022, year: 2022))

        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)in2023", model: 2023, year: 2022))

        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)in2024", model: 2024, year: 2022))
/*
 configs.append(MassTestConfig(path: "\(LKFTest.rootPath)inamb2019", stamm: "\(LKFTest.rootPath)in2019", model: 2019, year: 2019))
        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)inamb2020", stamm: "\(LKFTest.rootPath)in2020", model: 2020, year: 2019))
 
        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)inamb2021", stamm: "\(LKFTest.rootPath)in2021", model: 2021, year: 2019))
*/
        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)inamb2022", stamm: "\(LKFTest.rootPath)inamb2022", model: 2022, year: 2022))
        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)inamb2023", stamm: "\(LKFTest.rootPath)inamb2023", model: 2023, year: 2022))

        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)inamb2024", stamm: "\(LKFTest.rootPath)inamb2024", model: 2024, year: 2022))

        return configs
    }()
    
    static let adhoc_configs : [MassTestConfig] = {
        LKFContext.unloadStamm()
        var configs = [MassTestConfig]()
        configs.append(MassTestConfig(path: "\(LKFTest.rootPath)bernie/K6_Data", stamm: "\(LKFTest.rootPath)bernie", model: 2021, year: 2021))
        return configs
    }()
    
    override func setUp() {
        // Put setup code here. This method is called before the invocation of each test method in the class.
    }

    override func tearDown() {
        // Put teardown code here. This method is called after the invocation of each test method in the class.
    }
    
    func testMass() {
       /*
        for c in lkfcmasstests.adhoc_configs {
            c.contextFlags = 8388608
            c.packageSize = 5000
            c.test(testOutput: true, diff: .disable)
        }
         */
    
        if MassTestConfig.enableMassTest {
            for c in lkfcmasstests.configs {
                c.errorFiles = true
                c.test()
            }
        }
        else {
            XCTAssert(false, "Mass Test deactivated")
        }
    }
}
